@@ -16,6 +16,7 @@ from time import sleep
 from billiard.common import restart_state
 from billiard.exceptions import RestartFreqExceeded
 from kombu.asynchronous.semaphore import DummyLock
+from kombu.exceptions import ContentDisallowed, DecodeError
 from kombu.utils.compat import _detect_environment
 from kombu.utils.encoding import bytes_t, safe_repr
 from kombu.utils.limits import TokenBucket
@@ -263,49 +264,44 @@ class Consumer(object):
     def _update_qos_eventually(self, index):
         return (self.qos.decrement_eventually if index < 0
                 else self.qos.increment_eventually)(
-            abs(index) * self.prefetch_multiplier)
+                    abs(index) * self.prefetch_multiplier)
 
     def _limit_move_to_pool(self, request):
         task_reserved(request)
         self.on_task_request(request)
 
-    def _on_bucket_wakeup(self, bucket, tokens):
-        try:
-            request = bucket.pop()
-        except IndexError:
-            pass
-        else:
-            self._limit_move_to_pool(request)
-            self._schedule_oldest_bucket_request(bucket, tokens)
+    def _schedule_bucket_request(self, bucket):
+        while True:
+            try:
+                request, tokens = bucket.pop()
+            except IndexError:
+                # no request, break
+                break
 
-    def _schedule_oldest_bucket_request(self, bucket, tokens):
-        try:
-            request = bucket.pop()
-        except IndexError:
-            pass
-        else:
-            return self._schedule_bucket_request(request, bucket, tokens)
+            if bucket.can_consume(tokens):
+                self._limit_move_to_pool(request)
+                continue
+            else:
+                # requeue to head, keep the order.
+                bucket.contents.appendleft((request, tokens))
 
-    def _schedule_bucket_request(self, request, bucket, tokens):
-        bucket.can_consume(tokens)
-        bucket.add(request)
-        pri = self._limit_order = (self._limit_order + 1) % 10
-        hold = bucket.expected_time(tokens)
-        self.timer.call_after(
-            hold, self._on_bucket_wakeup, (bucket, tokens),
-            priority=pri,
-        )
+                pri = self._limit_order = (self._limit_order + 1) % 10
+                hold = bucket.expected_time(tokens)
+                self.timer.call_after(
+                    hold, self._schedule_bucket_request, (bucket,),
+                    priority=pri,
+                )
+                # no tokens, break
+                break
 
     def _limit_task(self, request, bucket, tokens):
-        if bucket.contents:
-            return bucket.add(request)
-        return self._schedule_bucket_request(request, bucket, tokens)
+        bucket.add((request, tokens))
+        return self._schedule_bucket_request(bucket)
 
     def _limit_post_eta(self, request, bucket, tokens):
         self.qos.decrement_eventually()
-        if bucket.contents:
-            return bucket.add(request)
-        return self._schedule_bucket_request(request, bucket, tokens)
+        bucket.add((request, tokens))
+        return self._schedule_bucket_request(bucket)
 
     def start(self):
         blueprint = self.blueprint
@@ -571,8 +567,10 @@ class Consumer(object):
                         promise(call_soon, (message.reject_log_error,)),
                         callbacks,
                     )
-                except InvalidTaskError as exc:
+                except (InvalidTaskError, ContentDisallowed) as exc:
                     return on_invalid_task(payload, message, exc)
+                except DecodeError as exc:
+                    return self.on_decode_error(message, exc)
 
         return on_task_received
 
